<?php

/**
 * Gestion de l'importation des contours des subdivisions dans la table `spip_gis`
**/

/**
 * Fonction d'import de données dans la table `spip_gis`
 *
 * utilisée dans le panneau de maintenance (si GIS n'est pas actif à l'installation)
 * et lors de l'installation si l'environnement est ad hoc (GIS + GIS Géométries).
 *
 *     ```
 *     include_spip('base/importer_spip_gis');
 *     $maj['create'][] = array('importer_spip_gis'); //install
 *     importer_spip_gis(); // action (hors install)
 *     ```
 *
**/

function importer_spip_gis() {


	// s'assurer que la table `spip_subdivisions` est présente et peuplée, car les contours seront liés à cette table
	$nombre = sql_countsel("spip_subdivisions");
	if (!$nombre) { // la table est inexistante ou n'est pas peuplée
	 	spip_log("function importer_spip_gis. spip_subdivisions non.",'subdivisions.' . _LOG_INFO_IMPORTANTE);
		return;
	} else { // identifier l'id_subdivision des contours
		$ins = array(); 
		// les données le code ISO
		list($cles, $valeurs) = donnees_spip_gis();
		// on interroge `spip_subdivisions` pour retrouver l'id_subdivision
		foreach ($valeurs as $v) {
			$ins[]= $v[0];
		}
		// le cas des doubles contours, là où la région correspond au département
		$d = array(
			'FR-GUA' => 'FR-GP',
			'FR-MTQ' => 'FR-MQ',
			'FR-GUF' => 'FR-GF',
			'FR-LRE' => 'FR-RE',
			'FR-MAY' => 'FR-YT'
		);
		foreach ($d as $double) {
			$ins[]= $double;
		}
		
		$in = sql_in('iso_moi', $ins);

		$mon_id = array(); $completude = array();
		if ($lignes = sql_allfetsel('iso_moi, id_subdivision', 'spip_subdivisions', array($in))) {
			foreach ($lignes as $l) {
				$mon_id[$l['iso_moi']] = $l['id_subdivision'];
				$completude[] = $l['iso_moi'];
			}
		} else {
	 			spip_log("function importer_spip_gis. requete refusée.",
	 				'subdivisions.' . _LOG_ERREUR);
	 			return;
		}
		
 		// vérification de la complétude des données. As-t-on toutes les régions ?
		foreach ($valeurs as $v) {
			if (!in_array($v[0],$completude)) {
	 			spip_log("function importer_spip_gis. La région n°" . $v[0] 
	 				. " est absente de la nomenclature qui a peuplé spip_subdivision",
	 				'subdivisions.' . _LOG_INFO_IMPORTANTE);
	 		}
		}
	}
	
	// vérifier si le plugin Gis Géometrie, qui gérera les contours, est actif
	// s'il est là, GIS est là (necessite)
	$geom = test_plugin_actif('gisgeom');
	if (!$geom) {
	 	spip_log("function importer_spip_gis. plugin gis_geom non.",'subdivisions.' . _LOG_INFO);
		return;
	}

	// Ajouter les subdivisions géographiques à la liste des objets gérés par gis
	// Voir : gis_pipelines.php function gis_afficher_contenu_objet()
	// Attention, si l'utilisateur n'a pas du tout configurer GIS (choix de la couche...),
	// cet ajout à la liste ne suffit pas à mettre en oeuvre ses traitements.
 	$gis_config = lire_config('gis/gis_objets', array());
 	if (!in_array('spip_subdivisions', $gis_config)) { // mais s'il oublie de dire que subdivisions peut être géolocalisé on le fait pour lui.
		$gis_config[] = 'spip_subdivisions';
		ecrire_config('gis/gis_objets', $gis_config);
	}
	
	// confier à gis les contours des subdivisions
	$table = 'spip_gis';

	/**
	* S'assurer que nous puissions retrouver nos contours en cas d'actualisation de ces données.
	* Différentes solutions :
	* 1. Une liaison avec un rôle, serait naturelle ; mais la colonne est absence de spip_gis_liens
	* 2. L'ajout d'une colonne rôle à la table de liaison demande le plugin rôle. 
	*    L'utilisateur définira un rôle : le GeoJSON pourra être 'définition' (de l'objet) ou 'élément' (de l'objet)
	* 3. placer id_gis la configuration avec une valeur sérializée serait sous-estimer le nombre de contours possibles.
	* 4. La création d'une table de traçage, en attentant le rôle sur la liaison, serait abrupt
	* 5. on 'laisse faire' et on offre un tableau de visualisation de objets GIS liés qui permettra au Webmestre d'actualiser.
	* La version actuelle du plugin Subdivisions se retire derrière cette dernière option.
	*/
	 
	// nom_du_champ_source => nom_du_champ_destination
	// mettre vide la destination ou supprimer la ligne permet de ne pas importer la colonne.

	$correspondances = array(
		//'id_gis' => 'id_gis', 
		'titre' => 'titre', 
		'descriptif' => 'descriptif', 
		'lat' => 'lat', 
		'lon' => 'lon', 
		'geo' => 'geo', 
		'zoom' => 'zoom', 
		'type' => 'type', 
		'adresse' => 'adresse', 
		'pays' => 'pays', 
		'code_pays' => 'code_pays', 
		'region' => 'region', 
		'departement' => 'departement', 
		'ville' => 'ville', 
		'code_postal' => 'code_postal', 
		'color' => 'color', 
		'weight' => 'weight', 
		'opacity' => 'opacity', 
		'fillcolor' => 'fillcolor', 
		'fillopacity' => 'fillopacity'
	);

	// transposer les donnees dans la nouvelle structure
	$inserts = array();

	// on remet les noms des cles dans le tableau de valeur
	// en s'assurant de leur correspondance au passage
	if (is_array($valeurs)) {
		foreach ($valeurs as $v) {
			$i = array();
			$iso_moi = $v[0];
			foreach ($v as $k => $valeur) {
				$cle = $cles[$k];
				if (isset($correspondances[$cle]) and $correspondances[$cle]) {
					$i[ $correspondances[$cle] ] = $valeur;
					if (($valeur == '') and ($correspondances[$cle]=='geo')) {
					// Récupérer la valeur du champ geo au format WKT
						include_spip('action/ajouter_documents');
						$id_subdivision = $mon_id[$v[0]];
						$id_cog = $v[1];
						$fichier = _DIR_PLUGIN_SUBDIVISIONS.'base/json/region_'.$id_cog.'.json';
						$import = '';
						lire_fichier($fichier, $donnees);
						if ($donnees) {
 							spip_log("Fonction importer_spip_gis. Région COG n°$id_cog. $fichier a des données.",
 								'subdivisions.' . _LOG_DEBUG);
							$f = find_in_path(_DIR_LIB_GEOPHP.'geoPHP.inc', '', true);
							if ($f) {
								$geometry = geoPHP::load($donnees, 'json');
								$i['geo'] = $geometry->out('json');
								// renseigner les coordonnées de l'objet à partir de son centroid
								$centroid = $geometry->getCentroid();
								$i['lat']= $centroid->getY();
								$i['lon']= $centroid->getX();
								$mon_type = $geometry->geometryType();
								$i['type'] = $mon_type;
	 							spip_log("Fonction importer_spip_gis. $fichier décodé. Centre "
	 								. $i['lat']."/".$i['lon'].". Forme : $mon_type.",
 									'subdivisions.' . _LOG_DEBUG);
								include_spip('gisgeom_fonctions');
								$wkt = json_to_wkt($i['geo']);
								// convertir le WKT en binaire avant l'insertion
								$i['geo'] = sql_getfetsel("GeomFromText('$wkt')");
								if ($id_gis = sql_insertq('spip_gis', $i)) {
 									spip_log("Fonction importer_spip_gis. la forme id_gis n°$id_gis est liée avec "
 										. $v[0] . " (id_subdivision n°$id_subdivision).",
										'subdivisions.' . _LOG_INFO
									);
									sql_insertq('spip_gis_liens', array(
										'id_gis' => $id_gis,
										'objet' => 'subdivision',
										'id_objet' => $id_subdivision,
										)
									);
									// cas des doubles contours (région=département)
									if (in_array($v[0], array('FR-GUA','FR-MTQ','FR-GUF','FR-LRE','FR-MAY'))) {
										sql_insertq('spip_gis_liens', array(
											'id_gis' => $id_gis,
											'objet' => 'subdivision',
											'id_objet' => $mon_id[$d[$v[0]]]
											)
										);
	 									spip_log("Fonction importer_spip_gis. Double liaison de la forme id_gis n°$id_gis "
	 										. "avec " . $v[0] . " (id_subdivision n°$id_subdivision) et " 
	 										. $d[$v[0]] . " (id_subdivision n°"
	 										. $mon_id[$d[$v[0]]] . ").",
 											'subdivisions.' . _LOG_INFO
 										);
									}
								}
							}
						}
					}
				}
			}
		}
	}
}


/**
 * Donnees de la table spip_gis
 * Définition des tracés des régions française, pour l'instant.
 * et quelques informations complétant le peuplement de Nomenclatures
**/
function donnees_spip_gis() {

	$cles = array('iso_moi', 'cog', 'titre', 'descriptif', 'lat', 'lon', 'geo', 'zoom', 'type', 'adresse', 'pays', 'code_pays', 'region', 'departement', 'ville', 'code_postal', 'color', 'weight', 'opacity', 'fillcolor', 'fillopacity');

	$valeurs = array(
		array('FR-GUA', '1' ,'Guadeloupe', 'Contour du Département et de la Région de la Guadeloupe', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-MTQ', '2', 'Martinique', 'Contour du Département et de la Région de la Martinique', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-GUF', '3', 'Guyane', 'Contour du Département et de la Région de la Guyane', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-LRE', '4', 'Réunion', 'Contour du Département et de la Région de la Réunion', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-MAY', '6', 'Mayotte', 'Contour du Département et de la Région de Mayotte', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-IDF',"11", 'Île-de-France', "Contour de la Région d'Île-de-France", '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-CVL', "24",'Centre-Val de Loire', 'Contour de la Région Centre-Val de Loire', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-BFC', "27",'Bourgogne-France-Comté', 'Contour de la Région Bourgogne-France-Comté', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-NOR', "28",'Normandie', 'Contour de la Région Normandie', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-HDF', "32",'Hauts-de-France', 'Contour de la Région Hauts-de-France', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-GES', "44",'Grand Est', 'Contour de la Région Grand Est', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-PDL', "52",'Pays de la Loire', 'Contour de la Région Pays de la Loire', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-BRE', "53",'Bretagne', 'Contour de la Région Bretagne', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-NAQ', "75", 'Nouvelle-Aquitaine', 'Contour de la Région Nouvelle-Aquitaine', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-OCC', "76",'Occitanie', 'Contour de la Région Occitanie', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-ARA', '84', 'Auvergne-Rhône-Alpes', 'Contour de la Région Auvergne-Rhône-Alpes', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-PAC', '93', "Provence-Alpes-Côte d'Azur", "Contour de la Région Provence-Alpes-Côte d'Azur", '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		array('FR-COR','94', 'Corse', 'Contour de la Région Corse', '', '','', '8', '', '', '', '', '', '', '', '', '', '', '', '', ''),
		);


	return array($cles, $valeurs);
}